/**************************************************************

File Name:	Complex_Vector_Add.asm
		
Date Modified:	02/17/99	RFG		original version
				05/01/00	RFG		modified format
				07/13/00	RFG		verified on silicon

Description:
			Subroutine that implements a Complex Vector Addition
			given two complex input vectors.

			Equation:	Z(n) = X(n) + Y(n)
						Z = (Xr + jXi) + (Yr + jYi) = (Xr+Yr) + j(Xi+Yi) 

Calling Parameters:
			b0,i0 = address of x input
			b8,i8 = address of y input
			b1,i1 = address of output
			r2 = number of samples (N) + 1
			l0,l1,l8 = 0
			m0,m8 = 2
			
Assumptions:
			All arrays must start on even address boundaries.
			All arrays must have an even number 32-bit word length (zero pad if necessary).
			X input is stored in Block 1 interleaved as real(0), imag(0), real(1), imag(1).
			Y input is stored in Block 0 interleaved as real(0), imag(0), real(1), imag(1).
			Ouput is stored in Block 1 interleaved as real(0), imag(0), real(1), imag(1).
			Instructions are stored in Block 0.

Return Values:
			b1 points to the output array

Registers Affected:
			f0,s0,f4,s4,f8,s8
			i0,i1,i8

Cycle Count:
			7 + (samples*2) + 2 cache misses
			
Memory Usage:
			Instructions Words (48-bits):
				7 instruction words
			
			Data Words (16, 32, 40, or 64-bits):
				2 * Number of samples locations for the x input buffer (32-bits)
				2 * Number of samples locations for the y input buffer (32-bits)
				2 * Number of samples + 4 locations for the output buffer (32-bits)

**************************************************************/

#include    "def21364.h"				/* Symbol Definition File */

.global cx_vec_add;

/* program memory code */
.section/pm seg_pmco;

cx_vec_add:    
	bit set MODE1 PEYEN;				/* SIMD enabled */
		
	lcntr=r2, do add until lce;
		f0=dm(i0,m0),	f4=pm(i8,m8);	/* f0 = Xr, s0 = Xi, f4 = Yr, s4 = Yi */
add:	f8=f0+f4,		dm(i1,m0)=f8;	/* f8 = Xr + Yr, s8 = Xi + Yi, store result */

	rts (db);
	bit clr MODE1 PEYEN;				/* SIMD disabled */
	dm(i1,m0)=f8;						/* store last result */
